package com.cesgroup.kingkong.interceptor;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.internal.util.AlipaySignature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cesgroup.common.utils.ApiResponse;
import com.cesgroup.common.utils.ApiResponseEnum;
import com.cesgroup.common.utils.ApiResultUtil;
import com.cesgroup.kingkong.entity.IPCheckEntity;
import com.cesgroup.kingkong.entity.SecretKeyEntity;
import com.cesgroup.kingkong.service.IPCheckService;
import com.cesgroup.kingkong.service.SecretKeyService;
import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @program: kyb-search
 * @description: 拦截器
 * @author: Wanjianbin
 * @create: 2019-08-27 09:48
 **/
@Slf4j
@Component
public class TokenInterceptor implements HandlerInterceptor {

    @Autowired
    private IPCheckService ipCheckService;
    @Autowired
    private SecretKeyService secretKeyService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        log.info("--------------------HTTP BEGIN, PREHANDLE BEGIN--------------------");
        log.info("--------------------REQUEST FROM: {}", ApiResultUtil.getUserIP(request));
        log.info("--------------------REQUEST URI IS: {}", request.getRequestURI());
        log.info("--------------------PARAMS BEGIN--------------------");
        Map<String, String[]> paramsMap = request.getParameterMap();
        Map<String, String> params = new HashMap<>();
        for (Map.Entry<String, String[]> entry : paramsMap.entrySet()){
            String[] values = entry.getValue();
            StringBuilder sb = new StringBuilder();
            for (String value : values) {
                sb.append(value);
            }
            log.info(entry.getKey() + " : " + sb.toString());
            params.put(entry.getKey(), sb.toString());
        }
        log.info("--------------------PARAMS END----------------------");

        // 免校验接口
        if (hasPermission(handler)){
            log.info("NO TOKENINTERCEPTOR, GOING TO CONTROLLER");
            return true;
        }

        ApiResponse apiResponse =null;
        String token = request.getHeader("token");
        if(StringUtil.isEmpty(token)){
            apiResponse = ApiResultUtil.getApiResponse(null, ApiResponseEnum.AUTH_TOKEN_ERROR);
        }else {
            IPCheckEntity ipCheckEntity = ipCheckService.getOne(new QueryWrapper<IPCheckEntity>().eq("token", token));
            if(ipCheckEntity != null){
                // 校验token是否过期
                if(checkToken(ipCheckEntity)){
                    if (checkIp(request, token)){
                        if (checkSign(params)){
                            IPCheckEntity entity = new IPCheckEntity();
                            entity.setLastCheckTime(new Date().getTime());
                            this.ipCheckService.update(entity, new UpdateWrapper<IPCheckEntity>().eq("token", token));
                            return true;
                        }else {
                            apiResponse = ApiResultUtil.getApiResponse(null, ApiResponseEnum.AUTH_SIGN_ERROR);
                        }
                    }else {
                        apiResponse = ApiResultUtil.getApiResponse(null, ApiResponseEnum.AUTH_IP_ERROR);
                    }

                }else {
                    apiResponse = ApiResultUtil.getApiResponse(null, ApiResponseEnum.TOKEN_OVERTIME_ERROR);
                }
            }else {
                apiResponse = ApiResultUtil.getApiResponse(null, ApiResponseEnum.AUTH_TOKEN_ERROR);
            }
        }

        response.setCharacterEncoding("utf-8");
        responseMessage(response,response.getWriter(),apiResponse);
        return false;
    }

    private boolean checkToken(IPCheckEntity ipCheckEntity){
        Long lastTokenTime = ipCheckEntity.getLastTokenTime();
        Date now = new Date();
        long diff = now.getTime() - lastTokenTime;
        if(diff < 1000*60*60*24){
            return true;
        }

        return false;
    }

    /** 
    * @Description: 校验IP白名单
    * @Param: [request] 
    * @return: boolean 
    * @Author: Wanjianbin
    * @Date: 2019/8/27 
    */
    private boolean checkIp(HttpServletRequest request, String token) {
        String remoteAddr = ApiResultUtil.getUserIP(request);
        int i = ipCheckService.getBaseMapper().selectCount(new QueryWrapper<IPCheckEntity>().eq("ip_address", remoteAddr).eq("token", token));
        if(i < 1){
            return false;
        }
        return true;
    }

    /**
    * @Description: 校验签名
    * @Param:
    * @return:
    * @Author: Wanjianbin
    * @Date: 2019/8/27
    */
    private boolean checkSign(Map params) throws Exception{
        String sign = MapUtils.getString(params, "sign");
        if (StringUtils.isEmpty(sign)){
            return false;
        }

        SecretKeyEntity secretKeyEntity = secretKeyService.getById(1);
        if(secretKeyEntity == null){
            log.info("-------------------- secretKey SELECT ERROR --------------------");
            return false;
        }
        String publicKey = secretKeyEntity.getPublicKey();

//        params.remove("sign");
//        String paramsString = JSON.toJSONString(params);
        if(!AlipaySignature.rsaCheckV2(params, publicKey, "UTF-8")){
            log.info("-------------------- HTTP END, TOKEN ERROR : {}--------------------", sign);
            return false;
        }
        return true;
    }

    /**
     * 是否有权限
     *
     * @param handler
     * @return
     */
    private boolean hasPermission(Object handler) {
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;
            // 获取方法上的注解
            RequiredPermission requiredPermission = handlerMethod.getMethod().getAnnotation(RequiredPermission.class);
            // 如果方法上的注解为空 进行拦截
            if (requiredPermission == null) {
                return false;
            }

        }
        return true;
    }

    /**
     * 返回信息给客户端
     *
     * @param response
     * @param out
     * @param apiResponse
     */
    private void responseMessage(HttpServletResponse response, PrintWriter out, ApiResponse apiResponse) {
        response.setContentType("application/json; charset=utf-8");
        out.print(JSONObject.toJSONString(apiResponse));
        out.flush();
        out.close();
    }
}
